#!/usr/local/bin/perl -w

#--------------------------------------------------------------------------------
#                                   Perldoc
#--------------------------------------------------------------------------------

=head1 NAME

nameconflate.pl

=head1 SYNOPSIS

 nameconflate.pl --words "Brad Pitt","Russell Crowe" --inpformat giga INPUT
OR
 nameconflate.pl --regexs "/(President )?(George )?(W. )?Bush/","/(Prime Minister |PM )?(Tony )?Blair/" --inpformat giga INPUT
OR
 nameconflate.pl --regexsfile REGEXSFILE --inpformat giga INPUT

=head1 DESCRIPTION

nameconflate.pl is a data creation utility for word sense 
disambiguation/discrimination experiments. 
In particular, we have used this to test the SenseClusters 
package. (http://senseclusters.sourceforge.net) 

nameconflate can also be viewed as a document format converter utility 
because it accepts input files in either English GigaWord format or plain 
text format and converts it to SENSEVAL2 format.

The main speciality of nameconflate is its capability to take given 
words or patterns and mask each of occurrence of the given words or 
words matching the given patterns in the input data and thus 
artificially create ambiguity.

For example, one could create an artificial ambiguity for the names 
"Brad Pitt" and "Russell Crowe" via the following command:

 nameconflate.pl --words "Brad Pitt","Russell Crowe" --inpformat giga INPUT
 
The output in this case might look like this (at the STDOUT):
 <?xml version="1.0" encoding="iso-8859-1" ?>
 <corpus lang='english'>

 <lexelt item="B_R">
 ...
 <instance id="9" docsrc="NYT20020607.0232">
 <answer instance="9" senseid="Brad_Pitt"/>
 <context>
few years back. The new film's premiere party at Universal Studios drew a 
certain co-star from another Damon flick. His ``Ocean's Eleven'' partner 
in crime <head>B_R</head> (still super scruffy for an upcoming role) brought 
his wife, Jennifer Aniston, who gave Camryn Manheim a big hug and eagerly 
posed with a few
 </context>
 </instance>

 <instance id="13" docsrc="NYT20020613.0159">
 <answer instance="13" senseid="Russell_Crowe"/>
 <context>
was made. During preproduction, Howard went to Princeton, N.J., to meet his 
somewhat reticent subject on his home turf and to record background film for 
<head>B_R</head> who plays the gradually aging Nash from his student days 
through his winning of a Nobel Memorial Prize in economic science in 1994. 
All the
 </context>
 </instance>
 ...

 </lexelt>
 </corpus>

Instead of looking for exact occurrences of given words in the input
text, nameconflate can also search for patterns specified using Perl
regular expressions. So to mask each of the following variation 
of the name George Bush:
 President George W. Bush
 President George Bush
 President Bush
 Bush

and to mask each of the following variation of the name Tony Blair:
 Prime Minister Tony Blair
 PM Tony Blair
 Tony Blair
 Blair

we could use following nameconflate command:
 nameconflate.pl --regexs "/(President )?(George )?(W. )?Bush/","/(Prime Minister |PM )?(Tony )?Blair/" --inpformat giga INPUT
OR
 nameconflate.pl --regexsfile REGEXFILE --inpformat giga INPUT

NOTE: In the second comand the regular expressions are specified in 
the file REGEXSFILE.
Contents of Sample REGEXSFILE:
 /(President )?(George )?(W. )?Bush/
 /(Prime Minister |PM )?(Tony )?Blair/

And a possible output (at STDOUT):
 <?xml version="1.0" encoding="iso-8859-1" ?>
 <corpus lang='english'>
 
 <lexelt item="XYZ">

 <instance id="1" docsrc="NYT20020601.0015">
 <answer instance="1" senseid="sense0" org_word="Bush"/>
 <context>
major, independent player in counterterrorism investigations, however, 
making it more difficult to coordinate counter-terrorism efforts on a 
bureauwide basis. Meanwhile, officials said that after the <head>XYZ</head> 
administration came into office, top Justice Department officials did not 
initially see the urgent need to upgrade counterterrorism. In August, 
officials said, the bureau's acting
 </context>
 </instance>

 <instance id="5" docsrc="NYT20020601.0021">
 <answer instance="5" senseid="sense0" org_word="President Bush"/>
 <context>
it back into balance. But the question making its way through the 
Pakistani capital is whether the president, Pervez Musharraf, has 
as much power as <head>XYZ</head> implies. At the same time, many in
India are wondering whether General Musharraf, who is known as a 
hawk, really wants to rein in Pakistan.
 </context>
 </instance>

 <instance id="122" docsrc="NYT20020601.0188">
 <answer instance="122" senseid="sense1" org_word="Blair"/>
 <context>
for three days as a prank or, as the plan's mastermind puts it, 
``an experiment in real life''? Think ``Lord of the Flies'' 
meets ``The <head>XYZ</head> Witch Project.'' Last year the 
reviewer King Kaufman called this a ``chilling'' little novel 
``about that most elemental of human fears, being buried alive.''
 </context>
 </instance>

 <instance id="237" docsrc="NYT20020603.0112">
 <answer instance="237" senseid="sense1" org_word="Prime Minister Tony Blair"/>
 <context>
Visitors, said that 46 current and 165 former heads of foreign 
governments have engaged in the ``hand-shake diplomacy'' of U.S. 
exchange programs. Alumni include British <head>XYZ</head> and 
Hamid Karzai, head of Afghanistan's interim government. If the 
goal of bringing Muslim opinion leaders to the United States is 
persuading them to agree
 </context>
 </instance>
 ...

 </lexelt>
 </corpus>

When using --regexs/--regexsfile option instead of --words:
 1. The answer tag contains an extra field "org_word" which stands 
for "original word" and its value specifies the exact word that 
matched one of the given patterns. 
 2. Each of the specified regex patterns is internally assigned an 
identifier based on its position in the argument list and this id 
along with the string "sense" is used to create the unique senseid 
for each given pattern and is used in the answer tag.

For usage information, type 'nameconflate.pl --help' at the command prompt.

=head1 Some additional information:

The English GigaWord format looks like...

 <DOC id="AFE19940512.0005" type="story" >
 <HEADLINE>
  Chinese dissidents in US favor partial revoking of MFN 
 </HEADLINE>
 <DATELINE>
 WASHINGTON, May 12 (AFP) 
 </DATELINE>
 <TEXT>
 <P>
 Chinese dissidents in the United States generally 
 favor a partial withdrawal of Beijing's privileged
 trading status targeting state-owned firms, not 
 complete revocation, dissident leaders said 
 here Thursday.
 </P>
 <P>
 There are differing views among the dissident
 community in the United States on the best way to
 advance human rights in China, but "an  
 overwhelming majority do agree on the middle 
 policy of targeted revocation," Zhao Haiching,
 the president of the National Council for 
 Chinese Affairs (NCCA) told a press conference.
 </P>
 </TEXT>
 </DOC>

=head1 AUTHOR

Anagha Kulkarni, kulka020@d.umn.edu, University of Minnesota, Duluth

Ted Pedersen, tpederse@d.umn.edu, University of Minnesota, Duluth

=head1 BUGS

=head1 SEE ALSO

 SenseClusters     :  http://sourceforge.net/projects/sensecluster

 English GigaWord  :  http://wave.ldc.upenn.edu/Catalog/CatalogEntry.jsp?catalogId=LDC2003T05

 SENSEVAL2         :  http://www.sle.sharp.co.uk/senseval2/  

=head1 COPYRIGHT

Copyright (C) 2005-2006, Ted Pedersen and Anagha Kulkarni

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, 
USA.

=cut


#------------------------------------------------------------------------------------------
#                                          Change Log
#------------------------------------------------------------------------------------------
# Date: 01/17/2005            Implementor: AKK              Version:0.10
# Extended the program  to conflate n words instead of 2 words.
# Status - complete
#
# Date: 01/18/2005            Implementor: TDP              Version:0.11
# modified perldoc slightly, removed unused variable
# Status - complete
#
# Date: 01/18/2005            Implementor: AKK              Version:0.12
# Made changes for removing warnings and turned the -w mode ON.
# Status - complete
#
# Date: 02/09/2005            Implementor: AKK              Version:0.13
# Instead of chomping the newlines from the input data we now replace 
# them by a space which avoids unexpected conflations.
# Status - complete
#
# Date: 07/24/2005            Implementor: AKK              Version:0.14
# Now supports plain text input file(s) along with GigaWords formated input
# files.
# Note: Only one format (plain/gigaword) can be used at a time.
# Status - complete
#
# Date: 01/20/2006            Implementor: AKK              Version:0.15
# 1. Added new option for specifying the starting number for instances.
# 2. Added new option for specifying the langauge of the input corpus and 
#    thus of the (senseval2) output .
# 3. Added new option for instructing to use one line per context 
#    i.e. to not cross line boundary.
# Status - complete
#
# Date: 04/09/2006            Implementor: AKK              Version:0.16
# 1. Added new option to support regular expressions instead of specifying words.
# # Status - complete

#-----------------------------------------------------------------------------------------
#                                Command Line Processing
#-----------------------------------------------------------------------------------------

use Getopt::Long;

# first check if no commandline options have been provided... in which case
# print out the usage notes
if ( $#ARGV == -1 )
{
    &minimalUsageNotes();
    exit;
}

# now get the options
my $result = GetOptions( "verbose", "version", "help", "words=s", "window=i","fullname","inpformat=s","instnum=i","lang=s","linecontext","regexs=s","regexsfile=s");

my @regexs = ();

# if help has been requested, print out help
if ( defined $opt_help )
{
    $opt_help = 1;
    &showHelp();
    exit;
}

# if version has been requested, show version!
if ( defined $opt_version )
{
    $opt_version = 1;
    &showVersion();
    exit;
}

my $obfusWord;
my $inpWords;

#Check if --words and --regex both specified
if(defined $opt_words && (defined $opt_regexs || defined $opt_regexsfile))
{
    print STDERR "The options --words and --regexs/--regexsfile cannot be used together.\n"; 
	print STDERR "Please use either --words or --regexs/--regexsfile.\n";
    askHelp();
    exit;	
}

#Check if words specified
if(!defined $opt_words && !defined $opt_regexs && !defined $opt_regexsfile)
{
    print STDERR "Please specify either --words or --regexs/--regexsfile.\n";
    askHelp();
    exit;	
}
elsif(defined $opt_words)
{
    @words = split(/,/,$opt_words);
    if($#words == -1)
    {
        print STDERR "Value for WORDS incorrect or not specified!\n";
        askHelp();
        exit;	
    }
    
	my $i = 0;
	for($i = 0; $i <= $#words; $i++)
	{
		#for regex matching later
		$inpWords .= '\b' . $words[$i] . '\b|';
	}
	$inpWords = substr($inpWords,0,length($inpWords)-1);

    # if --fullname option is specified then use the complete target words to create 
    # the obfuscated word else use the initials of the target words. 
    if(defined $opt_fullname)
    {
        # This is a work-around for avoiding the warning message that user gets:
        # Name "main::opt_fullname" used only once: possible typo at nameconflate.pl line ....
        $opt_fullname = $opt_fullname;

        my $i = 0;
        for($i = 0; $i <= $#words; $i++)
        {
            $inpWord = $words[$i];
            $inpWord =~ s/\s+/_/g;
            $obfusWord .= $inpWord . "_";
        }
        $obfusWord = substr($obfusWord,0,length($obfusWord)-1);
    }
    else
    {
        my $i = 0;
        for($i = 0; $i <= $#words; $i++)
        {
            $obfusWord .= substr($words[$i],0,1)."_";
        }
        $obfusWord = substr($obfusWord,0,length($obfusWord)-1);
    }
}
elsif(defined $opt_regexs)
{
	my $cnt = 0;

	while($opt_regexs=~/\/(.+?)\//g)
	{
		$regexs[$cnt] = $1;
		$cnt++;
	}

    if($#regexs == -1)
    {
        print STDERR "Value for --regexs incorrect or not specified!\n";
        askHelp();
        exit;	
    }

	my $i = 0;
	for($i = 0; $i <= $#regexs; $i++)
	{
		#for regex matching later
		$inpRegexs .= '\b' . $regexs[$i] . '\b|';
	}
	$inpRegexs = substr($inpRegexs,0,length($inpRegexs)-1);

	$obfusWord = "XYZ";
}
elsif(defined $opt_regexsfile)
{
	open(FP,"$opt_regexsfile") || die "Error opening regex file $opt_regexsfile.\n";

	if(-z FP)
	{
        print STDERR "$opt_regexsfile empty!\n";
        askHelp();
        exit;	
    }

	my $cnt = 0;
	while(<FP>)
	{
		chomp;
		s/^\s+//;       # skip leading spaces
		/^\/(.+?)\/$/;  # strip leading and following "/"
		$regexs[$cnt] = $1;
		$cnt++;
	}

	my $i = 0;
	for($i = 0; $i <= $#regexs; $i++)
	{
		#for regex matching later
		$inpRegexs .= '\b' . $regexs[$i] . '\b|';
	}
	$inpRegexs = substr($inpRegexs,0,length($inpRegexs)-1);

	$obfusWord = "XYZ";
}


#Check if window option NOT specified at the command line OR specified but with incorrect value eg. a string
if(!defined $opt_window)
{
    #if execution has made it till this point, it indicates that all other options are valid, 
    #but still if return value of result variable is 0, this indicates that there is some
    #problem with window option.
    if($result == 0)    
    {
        askHelp();
        exit;		
    }
    else
    {
        $windowSize = 25;                #set to default
    }
}
else
{
    #window option is specified and given valid value.
    if($opt_window < 0)
    {
        print STDERR "Value for WINDOW ($opt_window) incorrect!\n";
        askHelp();
        exit;		
    }
    $windowSize = $opt_window;
    chomp($windowSize);
}

my $inpfmt;
# Input files' format: plain (default) OR GigaWords
if(!defined $opt_inpformat || $opt_inpformat =~ /plain/i)
{
	$inpfmt = 0;
}
else #GigaWords
{
	$inpfmt = 1;
}

# Initialise the instance ID variable.
if(!defined $opt_instnum)
{
	$startinstID = 1;
	$instanceID = 1;
}
else
{
	$startinstID = $opt_instnum;
	$instanceID = $opt_instnum;
}

# Specify the language
if(!defined $opt_lang)
{
	$lang = "english";
}
else
{
	$lang = $opt_lang;
}

if(defined $opt_linecontext && $inpfmt == 1)
{
	print STDERR "\'--linecontext\' option can be used only if \'--inpformat plain\' specified.\n";
	askHelp();
	exit;		
}
 
# whats left in the command line are paths. go thru them and salvage all
# text files to be processed. the following function does just that, putting
# all useful files in @sourceFiles :o)
&getSourceFiles(@ARGV);

# check if a input file is getting interpreted as value for the preceding option, for which the user has not specified any value.
if (($#sourceFiles == -1 && defined $opt_words) || ($#sourceFiles == -1 && defined $opt_regexs) || ($#sourceFiles == -1 && defined $opt_regexsfile))
{
    print STDERR "Value for WORDn OR PATTERNn or REGEXSFILE incorrect or not specified!\n";
    askHelp();
    exit;	
}

# if not even one file found, complain and quit!
if ($#sourceFiles == -1)
{
    print STDERR "No input (INPUT) file supplied or no Valid (INPUT) file supplied!\n";
    askHelp();
    exit;
}

#--------------------------------------------------------------------------------------
#                                   Program logic
#--------------------------------------------------------------------------------------

foreach $inFile(@sourceFiles)
{
    # Open the input & output files
    open(INFILE, $inFile) || die "Unable to open input file in read mode!";

	# If input files in GigaWords format
	if($inpfmt)
	{
		if(defined $opt_words)
		{
			# Read a line & if it contain "<DOC" store the "id" in a variable.
			while($inStr = <INFILE>)
			{    
				if($inStr =~ /<DOC\s+id=\"(.+?)\"/)
				{
					$docID = $1;
				
					# read the input file till you get "<TEXT>" and discard all the line untill that
					until($inStr =~ /\<TEXT\>/)
					{
						$inStr = <INFILE>;
					}
					
					# read the input file till you get "</TEXT>" and store all the line in a variable.
					# remove the "<P>" & "</P>" tags and also the newlines
					$textStr = "";
					until($inStr =~ /\<\/TEXT\>/)
					{
						$inStr = <INFILE>;
						
						# removing the newlines were conflating the word before the new-line and after the new-line.
						# which is not expected thus instead of chomping the new-line now we replace it with a space.
						#chomp($inStr);
						$inStr=~s/\n+/ /;
						
						if($inStr !~ /\<P\>/ && $inStr !~ /\<\/P\>/)
						{
							$textStr = $textStr.$inStr;
						}
					}
					$textStr =~ s/\<\/TEXT\>//;
				
					#$textStr_Second = $textStr;  tdp removed, unused 
					
					#search for the word in the compiled string
					#if word is found store the window in a seperate variable
					
					#	while($textStr =~ /((\s+\S+){0,$windowSize})\s+(\b$sWord1\b)\s+((\S+\s+){0,$windowSize})/g)
					
					#	while($textStr =~ /((\s+\S+){0,$windowSize})\s+(\b$sWord1\b)/g)
					#            $preStr =~ /((\s*\S+\s*){0,$windowSize})$/;             
					#            $preWindow = $&;   
					
					$count = 0;
					while($count < @words)
					{   
						$wStr = $textStr;
						$sWord = $words[$count];
						
						#If the input word is a phrase, then the space(s) between the words needs to be replaced by '_'
						#because the label.pl program of SenseCluster package assumes, the senseids i.e. the original words
						#to be single word each and not a phrase of words.
						$senseid = $sWord;
						$senseid =~ s/\s+/_/g;
						
						while($wStr =~ /(\b$sWord\b)/g)
						{	
							$preStr = "$`";
							$postStr = "$'";
							
							@preStr = split(/\s+/,$preStr);
							
							if($preStr && @preStr>=$windowSize)
							{
								@temp = @preStr[@preStr-$windowSize..$#preStr];
								$preWindow = join " ",@temp;
							}
							elsif($preStr && @preStr > 0)
							{
								@temp = @preStr[0..$#preStr];
								$preWindow = join " ",@temp;
							}  
							else
							{
								$preWindow = "";
							}
							
							if($postStr =~ /\s+((\S+\s*){0,$windowSize})/)
							{						
								$postWindow = $&;
							}
							else
							{
								$postWindow = "";
							}
							
							$finalStr = $preWindow." <head>".$obfusWord."</head>".$postWindow;   
							
							if($instanceID eq $startinstID)
							{
								# Write the header part to the output file.
								print "<?xml version=\"1.0\" encoding=\"iso-8859-1\" ?>\n";
								print "<corpus lang=\'$lang\'>\n\n";
								print "<lexelt item=\"$obfusWord\">\n\n";
							}
							
							print "<instance id=\"$instanceID\" docsrc=\"$docID\">\n";
							print "<answer instance=\"".$instanceID++."\" senseid=\"".$senseid."\"/>\n";
							print "<context>\n";
							print "$finalStr\n";
							print "</context>\n";
							print "</instance>\n\n";
						} #while
						$count++;
					} #while
				} #if
			} #while
		} #if defined words 
		elsif(defined $opt_regexs || defined $opt_regexsfile) 
		{
			# Read a line & if it contain "<DOC" store the "id" in a variable.
			while($inStr = <INFILE>)
			{    
				if($inStr =~ /<DOC\s+id=\"(.+?)\"/)
				{
					$docID = $1;
				
					# read the input file till you get "<TEXT>" and 
					# discard all the line untill that
					until($inStr =~ /\<TEXT\>/)
					{
						$inStr = <INFILE>;
					}
					
					# read the input file till you get "</TEXT>" and 
					# store all the line in a variable.
					# remove the "<P>" & "</P>" tags and also the newlines
					$textStr = "";
					until($inStr =~ /\<\/TEXT\>/)
					{
						$inStr = <INFILE>;
						
						# removing the newlines were conflating the word before 
						# the new-line and after the new-line.
						# which is not expected thus instead of chomping the 
						# new-line now we replace it with a space.
						$inStr=~s/\n+/ /;
						
						if($inStr !~ /\<P\>/ && $inStr !~ /\<\/P\>/)
						{
							$textStr = $textStr.$inStr;
						}
					}
					$textStr =~ s/\<\/TEXT\>//;
				
					#$textStr_Second = $textStr;  tdp removed, unused 
					
					#search for the word in the compiled string
					#if word is found store the window in a seperate variable
					
					#	while($textStr =~ /((\s+\S+){0,$windowSize})\s+(\b$sWord1\b)\s+((\S+\s+){0,$windowSize})/g)
					
					#	while($textStr =~ /((\s+\S+){0,$windowSize})\s+(\b$sWord1\b)/g)
					#            $preStr =~ /((\s*\S+\s*){0,$windowSize})$/;             
					#            $preWindow = $&;   

					$count = 0;
					while($count <= $#regexs)
					{   
						$wStr = $textStr;
						$sWord = $regexs[$count];
						
						#If the input word is a phrase, then the space(s) 
						#between the words needs to be replaced by '_'
						#because the label.pl program of SenseCluster package 
						#assumes, the senseids i.e. the original words
						#to be single word each and not a phrase of words.

						$senseid = "sense$count";
					   	$senseid =~ s/\s+/_/g;
						
						while($wStr =~ /\b($sWord)\b/g)
						{	
							$org_word = $&;
							$preStr = "$`";
							$postStr = "$'";
							
							@preStr = split(/\s+/,$preStr);
							
							if($preStr && @preStr>=$windowSize)
							{
								@temp = @preStr[@preStr-$windowSize..$#preStr];
								$preWindow = join " ",@temp;
							}
							elsif($preStr && @preStr > 0)
							{
								@temp = @preStr[0..$#preStr];
								$preWindow = join " ",@temp;
							}  
							else
							{
								$preWindow = "";
							}
							
							if($postStr =~ /\s+((\S+\s*){0,$windowSize})/)
							{						
								$postWindow = $&;
							}
							else
							{
								$postWindow = "";
							}
							
							$finalStr = $preWindow." <head>".$obfusWord."</head>".$postWindow;   
							
							if($instanceID eq $startinstID)
							{
								# Write the header part to the output file.
								print "<?xml version=\"1.0\" encoding=\"iso-8859-1\" ?>\n";
								print "<corpus lang=\'$lang\'>\n\n";
								print "<lexelt item=\"$obfusWord\">\n\n";
							}
							
							print "<instance id=\"$instanceID\" docsrc=\"$docID\">\n";
							print "<answer instance=\"".$instanceID++."\" senseid=\"".$senseid."\" org_word=\"".$org_word ."\"/>\n";
							print "<context>\n";
							print "$finalStr\n";
							print "</context>\n";
							print "</instance>\n\n";
						} #while
						$count++;
					} #while
				} #if
			} #while
		} #elsif defined regexs 
	}#if --inpformat giga
	else #if input file format- plain 
	{
		if(defined $opt_linecontext) #if one line per context specified i.e. line boundary
		{	                         #crossing not allowed while creating the contexts.
			if(defined $opt_words)
			{
				my $intxt = "";
				while($intxt = <INFILE>)
				{
					chomp($intxt);
					
					# check if any of the words is encountered
					while($intxt =~ m/$inpWords/g)
					{
						$senseid = $&;
						$preStr = "$`";
						$postStr = "$'";
						
						#If the input word is a phrase, then the space(s) between the 
						#words needs to be replaced by '_' because the label.pl program 
						#of SenseCluster package assumes, the senseids i.e. the original words
						#to be single word each and not a phrase of words.
						$senseid =~ s/\s+/_/g;
						
						# select the context for the identified word
						@preStr = split(/\s+/,$preStr);
						
						if($preStr && @preStr>=$windowSize)
						{
							@temp = @preStr[@preStr-$windowSize..$#preStr];
							$preWindow = join " ",@temp;
						}
						elsif($preStr && @preStr > 0)
						{
							@temp = @preStr[0..$#preStr];
							$preWindow = join " ",@temp;
						}  
						else
						{
							$preWindow = "";
						}
						
						if($postStr =~ /\s+((\S+\s*){0,$windowSize})/)
						{						
							$postWindow = $&;
						}
						else
						{
							$postWindow = "";
						}
						
						# final compiled context
						$finalStr = $preWindow." <head>".$obfusWord."</head>".$postWindow;   
						
						#SENSEVAL2 formated header
						if($instanceID eq $startinstID)
						{
							# Write the header part to the output file.
							print "<?xml version=\"1.0\" encoding=\"iso-8859-1\" ?>\n";
							print "<corpus lang=\'$lang\'>\n\n";
							print "<lexelt item=\"$obfusWord\">\n\n";
						}
						
						#SENSEVAL2 formated context conatining instanceid and the answer
						print "<instance id=\"$instanceID\">\n";
						print "<answer instance=\"".$instanceID++."\" senseid=\"".$senseid."\"/>\n";
						print "<context>\n";
						print "$finalStr\n";
						print "</context>\n";
						print "</instance>\n\n";
					}
				}
			} # if defined $opt_words
			elsif(defined $opt_regexs || defined $opt_regexsfile)
			{
				my $intxt = "";
				while($intxt = <INFILE>)
				{
					chomp($intxt);
					
					# check if any of the words is encountered
					while($intxt =~ m/($inpRegexs)/g)
					{
						$org_word = $&;
						$preStr = "$`";
						$postStr = "$'";

						my $i = 0;
						for($i = 0; $i <= $#regexs; $i++)
						{
							if($org_word =~ m/\b$regexs[$i]\b/)
							{
								$senseid = "sense$i";
								last;
							}
						}
						
						#If the input word is a phrase, then the space(s) between 
						#the words needs to be replaced by '_' because the label.pl 
						#program of SenseCluster package assumes, the senseids i.e. 
						#the original words to be single word each and not a phrase of words.
						$senseid =~ s/\s+/_/g;
						
						# select the context for the identified word
						@preStr = split(/\s+/,$preStr);
						
						if($preStr && @preStr>=$windowSize)
						{
							@temp = @preStr[@preStr-$windowSize..$#preStr];
							$preWindow = join " ",@temp;
						}
						elsif($preStr && @preStr > 0)
						{
							@temp = @preStr[0..$#preStr];
							$preWindow = join " ",@temp;
						}  
						else
						{
							$preWindow = "";
						}
						
						if($postStr =~ /\s+((\S+\s*){0,$windowSize})/)
						{						
							$postWindow = $&;
						}
						else
						{
							$postWindow = "";
						}
						
						# final compiled context
						$finalStr = $preWindow." <head>".$obfusWord."</head>".$postWindow;   
						
						#SENSEVAL2 formated header
						if($instanceID eq $startinstID)
						{
							# Write the header part to the output file.
							print "<?xml version=\"1.0\" encoding=\"iso-8859-1\" ?>\n";
							print "<corpus lang=\'$lang\'>\n\n";
							print "<lexelt item=\"$obfusWord\">\n\n";
						}
						
						#SENSEVAL2 formated context conatining instanceid and the answer
						print "<instance id=\"$instanceID\">\n";
						print "<answer instance=\"".$instanceID++."\" senseid=\"".$senseid."\" org_word=\"".$org_word ."\"/>\n";
						print "<context>\n";
						print "$finalStr\n";
						print "</context>\n";
						print "</instance>\n\n";
					}
				}
			}
		}
		else # one line per context NOT specified i.e. line boundary crossing 
		{    # allowed while creating the contexts

			if(defined $opt_words)
			{
				# read the input file
				while($inStr = <INFILE>)
				{    
					chomp($inStr);
					$inStr .= " ";
					$intxt .= $inStr;
				} # while !eof(inpfile) 
				
				# check if any of the words is encountered
				while($intxt =~ m/$inpWords/g)
				{
					$senseid = $&;
					$preStr = "$`";
					$postStr = "$'";
					
					#If the input word is a phrase, then the space(s) between the words needs to be replaced by '_'
					#because the label.pl program of SenseCluster package assumes, the senseids i.e. the original words
					#to be single word each and not a phrase of words.
					$senseid =~ s/\s+/_/g;
					
					# select the context for the identified word
					@preStr = split(/\s+/,$preStr);
					
					if($preStr && @preStr>=$windowSize)
					{
						@temp = @preStr[@preStr-$windowSize..$#preStr];
						$preWindow = join " ",@temp;
					}
					elsif($preStr && @preStr > 0)
					{
						@temp = @preStr[0..$#preStr];
						$preWindow = join " ",@temp;
					}  
					else
					{
						$preWindow = "";
					}
					
					if($postStr =~ /\s+((\S+\s*){0,$windowSize})/)
					{						
						$postWindow = $&;
					}
					else
					{
						$postWindow = "";
					}
					
					# final compiled context
					$finalStr = $preWindow." <head>".$obfusWord."</head>".$postWindow;   
					
					#SENSEVAL2 formated header
					if($instanceID eq $startinstID)
					{
						# Write the header part to the output file.
						print "<?xml version=\"1.0\" encoding=\"iso-8859-1\" ?>\n";
						print "<corpus lang=\'$lang\'>\n\n";
						print "<lexelt item=\"$obfusWord\">\n\n";
					}
					
					#SENSEVAL2 formated context conatining instanceid and the answer
					print "<instance id=\"$instanceID\">\n";
					print "<answer instance=\"".$instanceID++."\" senseid=\"".$senseid."\"/>\n";
					print "<context>\n";
					print "$finalStr\n";
					print "</context>\n";
					print "</instance>\n\n";
				} #while
			}
			elsif(defined $opt_regexs || defined $opt_regexsfile)
			{
				# read the input file
				$intxt = "";
				while($inStr = <INFILE>)
				{    
					chomp $inStr;
					$inStr .= " ";
					$intxt .= $inStr;
				} # while !eof(inpfile) 

				# check if any of the words is encountered
				while($intxt =~ m/($inpRegexs)/g)
				{					
					$org_word = $&;
					$preStr = "$`";
					$postStr = "$'";

					my $i = 0;
					for($i = 0; $i <= $#regexs; $i++)
					{
						if($org_word =~ m/\b$regexs[$i]\b/)
						{
							$senseid = "sense$i";
							last;
						}
					}

					#If the input word is a phrase, then the space(s) between 
					#the words needs to be replaced by '_' because the label.pl 
					#program of SenseCluster package assumes, the senseids i.e. 
					#the original words to be single word each and not a phrase of words.
					$senseid =~ s/\s+/_/g;
					
					# select the context for the identified word
					@preStr = split(/\s+/,$preStr);
					
					if($preStr && @preStr>=$windowSize)
					{
						@temp = @preStr[@preStr-$windowSize..$#preStr];
						$preWindow = join " ",@temp;
					}
					elsif($preStr && @preStr > 0)
					{
						@temp = @preStr[0..$#preStr];
						$preWindow = join " ",@temp;
					}  
					else
					{
						$preWindow = "";
					}

					if($postStr =~ /\s+((\S+\s*){0,$windowSize})/)
					{						
						$postWindow = $&;
					}
					else
					{
						$postWindow = "";
					}
					
					# final compiled context
					$finalStr = $preWindow." <head>".$obfusWord."</head>".$postWindow;   
					
					#SENSEVAL2 formated header
					if($instanceID eq $startinstID)
					{
						# Write the header part to the output file.
						print "<?xml version=\"1.0\" encoding=\"iso-8859-1\" ?>\n";
						print "<corpus lang=\'$lang\'>\n\n";
						print "<lexelt item=\"$obfusWord\">\n\n";
					}
					
					#SENSEVAL2 formated context conatining instanceid and the answer
					print "<instance id=\"$instanceID\">\n";
#					print "<answer instance=\"".$instanceID++."\" senseid=\"".$senseid."\"/>\n";

					print "<answer instance=\"".$instanceID++."\" senseid=\"".$senseid."\" org_word=\"".$org_word ."\"/>\n";
					print "<context>\n";
					print "$finalStr\n";
					print "</context>\n";
					print "</instance>\n\n";
				} #while
			}
		} #else --linecontext
	} #else --plain
}
	
if($instanceID > $startinstID)
{
	print "</lexelt>\n";
	print "</corpus>\n";
}
else
{
	print STDERR "The input file(s) does not not contain any of the target words, please try some other pair of words.\n";
	exit;
}


#--------------------------------------------------------------------------------------
#                                         Function definitions
#--------------------------------------------------------------------------------------
# function to output a minimal usage note when the user has not provided any
# commandline options
sub minimalUsageNotes
{
    print STDERR "Usage: nameconflate.pl [OPTIONS] --words WORD1,WORD2,WORD3,... INPUT [[, INPUT] ...]\n";
    print STDERR "OR\nUsage: nameconflate.pl [OPTIONS] --regexs PATTERN1,PATTERN2,PATTERN3,... INPUT [[, INPUT] ...]\n";	
    print STDERR "OR\nUsage: nameconflate.pl [OPTIONS] --regexsfile REGEXSFILE INPUT [[, INPUT] ...]\n";
    askHelp();
}

# function to output help messages for this program
sub showHelp
{
    print "\nUsage: nameconflate.pl [OPTIONS] --words WORD1,WORD2,WORD3,... INPUT [[, INPUT] ...]\n";
    print "OR\nUsage: nameconflate.pl [OPTIONS] --regexs PATTERN1,PATTERN2,PATTERN3,... INPUT [[, INPUT] ...]\n";	
    print "OR\nUsage: nameconflate.pl [OPTIONS] --regexsfile REGEXSFILE INPUT [[, INPUT] ...]\n\n";	
    print "This scripts accepts the input file(s) in either plain text (default) \n";
	print "or the GigaWord format and gives output in SENSEVAL2 format. The \n";
	print "program searches the input file(s) for the words or patterns (specified \n";
	print "by the user at the command line or in a file), masks each of their \n";
	print "occurrences and creates instances of max(1 + 2 * window size) words. \n";
	print "Each such instances contains only one occurrence of the masked word. \n";
	print "If input is a directory, then it is recursively searched for input files.\n\n";
	  
    print "OPTIONS:\n\n";
	  
    print "  --window WINDOW              The window size for each instance. \n";
    print "                               (default=25)\n\n";

    print "  --fullname                   The complete target words are used \n";
    print "                               to generate the obfuscated word. \n";
    print "                               If not specified uses initials of \n";
    print "                               the target words to create the \n";
    print "                               obfuscated word\n";
    print "                               NOTE: Can be used only with --words\n";
    print "                               i.e. cannot be used with --regexs.\n\n";

    print "  --inpformat FMT              The format of the input file(s). \n";
	print "                               FMT = plain (default) / giga\n\n";

	print "  --instnum NUM                The starting Instance Number/ID. \n";
    print "                               (default=1)\n\n";

	print "  --lang LANG                  The langauge of the input corpus and\n"; 
    print "                               thus of the (senseval2) output. \n";
	print "                               To be included in one of the xml tags \n";  
    print "                               (default=english)\n\n";

	print "  --linecontext                One line per context. \n";
	print "                               i.e. The line boundary is not to be\n";
	print "                               crossed during context creation.\n";
	print "                               (This option can be used only if \n";
	print "                               \"--inpformat plain\" specified). \n\n"; 
  
    print "  --verbose                    Outputs to stderr information about \n";
    print "                               current program status.\n\n";
	  
    print "  --version                    Prints the version number.\n\n";
	  
    print "  --help                       Prints this help message.\n\n";

    print "REQUIRED ARGUMENTS:\n\n";
	  
    print "  --words WORD1,WORD2,...      The comma seperated list of words to be \n";
    print "                               obfuscated.\n";
	print "                               Please do not insert spaces between the \n";
    print "                               WORD and the comma (,)\n";

    print "\n  OR\n\n";
  
    print "  --regexs PATTERN1,...        The comma seperated list of patterns to be \n";
    print "                               matched with words/phrases and then to be masked.\n";
	print "                               The patterns should be specified in Perl regular\n";
	print "                               expressions format.\n";
	print "                               Please do not insert spaces between the \n";
    print "                               PATTERN and the comma (,)\n\n";

    print "\n  OR\n\n";
  
    print "  --regexsfile REGEXSFILE      The list of patterns to be matched can also be \n"; 
	print "                               specified in a file (instead at command-line).\n";
	print "                               NOTE:Please specify one regex/line in the file.\n\n";

    print "  INPUT                        The input file/directory to be processed.\n";
    print "                               If input is a directory, then it is \n";
    print "                               recursively searched for input files.\n\n";
	  
}

# function to output the version number
sub showVersion
{
    print "nameconflate.pl      -        version 0.16\n";
    print "Copyright (C) 2005-2006, Ted Pedersen & Anagha Kulkarni\n";
    print "Date of Last Update 04/09/2006\n";
}

# function to output "ask for help" message when the user's goof up!
sub askHelp
{
    print STDERR "Type nameconflate.pl --help for help.\n";
}


# Function &getSourceFiles: function to take the command tail and
# return an array of text files to be used! while going thru the
# command line do the following processing:
#
# 1> if the string is a text file and can be opened, add it to the array.
# 2> if the string is a directory name, find all text files in that directory,
#    and append to array.
# 3> if the -r (recursive) option is set, go into all subdirectories of that
#    directory too, to do the above!

sub getSourceFiles
{
    # get the next commandline string...
    my $nextString = shift;
    $index = 0;
    
    while ( $nextString )
    {
        if ( !( -e $nextString ) )
        {
            # file doesn't exist... ignore!
	    
            if ( defined $opt_verbose ) { print STDERR "File $nextString does not exist!\n"; }
            $nextString = shift;
            next;
        }
	
    	if ( !( -r $nextString ) )
    	{
            # file can't be read... ignore!
            if ( defined $opt_verbose ) { print STDERR "File $nextString cant be read!\n"; }
            $nextString = shift;
            next;
    	}
    	
        if ( -d $nextString )
        {
            # this is a directory, go and search this directory for text files
            &directorySearch( $nextString );
            $nextString = shift;
            next;
        }
	
        if ( !( -T $nextString ) )
        {
            # file is not a text file... ignore!
            if ( defined $opt_verbose ) { print STDERR "$nextString is not a text file!\n"; }   
            $nextString = shift; 		
            next;
        }
	
        $sourceFiles[$index] = $nextString; 
        $index++;
        $nextString = shift;
    }
}

# function to (possibly recursively) search inside the given directory for
# text files
sub directorySearch
{
    my $directory = $_[0];
    
    # This is a work-around for avoiding the warning message that user gets:
    # Name "main::opt_recurse" used only once: possible typo at nameconflate.pl line ....
    if(defined $opt_recurse)
    {
        $opt_recurse = $opt_recurse;
    }

    opendir DIR, $directory || "Couldnt open directory $directory!\n";
    my @files = grep !/^\./, readdir DIR;
    @files = map "$directory/$_", @files;
    closedir DIR;
    
    my $file = "";
    
    foreach $file (@files)
    {
        if ( ( -d $file ) && ( defined $opt_recurse ) ) { &directorySearch($file); }
        if ( ( -T $file ) )
        {
            $sourceFiles[$index] = $file; 
            $index++;
        }
    }
}
